interface FetchOptions extends RequestInit {
  retry?: number;
  timeout?: number;
}

export default class FetchWrapper {
  static async fetch(url: RequestInfo, options: FetchOptions = {}): Promise<Response> {
    const mergedOptions: RequestInit = {
      ...{
        method: 'GET',
        headers: {
          'Content-Type': 'application/json',
        },
      },
      ...options,
    };

    let response: Response | unknown;
    let retries = options.retry || 0;

    while (true) {
      try {
        if (options.timeout ?? 0) {
          response = await Promise.race([
            fetch(url, mergedOptions),
            new Promise((_, reject) =>
              setTimeout(() =>
                reject(new Error(`Request timed out after ${options.timeout}ms`)),
                options.timeout)),
          ]);
        } else {
          response = await fetch(url, options);
        }
        if (!(response instanceof Response)) {
          throw response;
        }


        return response;

      } catch (error) {
        if (retries > 0) {
          retries--;
          console.debug(`Retrying fetch request... (${retries} retries left)`);
          await new Promise(resolve => setTimeout(resolve, 1000));
        } else {
          throw error;
        }
      }
    }
  }
}
